﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace dotMath2
{
    /// <summary>
    /// Token class:  The Token class represents a collection of 1 or more characters
    ///		that form a given set of singly definable characters.
    /// </summary>
    public class Token
    {
        private string _token;
        private CharType _charType;

        /// <summary>
        /// Equals(Object): Tests for equality with another token.  Also
        ///		handles the null case.
        /// </summary>
        /// <param name="sValue">string value representing the token being evaluated against.</param>
        /// <returns>bool where true means both objects are equal.</returns>
        public override bool Equals(Object value)
        {
            if (value == null)
                return false;


            string testValue = string.Empty;
            
            if (value is string) testValue = value as string;
            if (value is Token) testValue = (value as Token).ToString();

            return testValue == _token;
        }

        public override int GetHashCode()
        {
            if (string.IsNullOrEmpty(_token))
                return string.Empty.GetHashCode();

            return _token.GetHashCode();
        }

        /// <summary>
        /// operator == (Token, string): This overloaded operator checks for equality between a token and a string.
        /// </summary>
        /// <param name="sToken">Token - actual token object</param>
        /// <param name="sValue">string - token value to test against token object value</param>
        /// <returns></returns>
        public static bool operator ==(Token token, string value)
        {
            // the following tests are order dependent.  If both are null, eq is true.  
            // if only one is null the eq is false.


            if ( value == null)
                return false;

            if (token._token == value)
                return true;
            else
                return false;
        }

        /// <summary>
        /// operator != (Token, string): Overloaded operator that checks for inequality
        ///		between the token value and the passed string.
        /// </summary>
        /// <param name="sToken">Token object being test</param>
        /// <param name="sValue">string that is being compared with the Token object's string value</param>
        /// <returns>bool indicating true for inequality</returns>
        public static bool operator !=(Token token, string value)
        {
            if (value == null)
                return false;

            if (token._token != value)
                return true;
            else
                return false;
        }

        /// <summary>
        /// Token(string, CharType) constructor: A constructor that creates a new token object with the
        ///		passed token string and character type information.
        /// </summary>
        /// <param name="sToken">string representing the token value</param>
        /// <param name="chType">Character enumeration describing the type of data in the token.</param>
        public Token(string token, CharType chType)
        {
            if (string.IsNullOrEmpty(token))
                token = string.Empty;

            _token = token;
            _charType = chType;
        }

        /// <summary>
        /// Token() constructor:  creates an empty token
        /// </summary>
        public Token()
        {
            _token = "";
            _charType = CharType.CT_UNDEF;
        }

        /// <summary>
        /// Token(char, CharType) constructor: create a single character token with type info
        /// </summary>
        /// <param name="chToken">Single character token value</param>
        /// <param name="chType">Token Type information</param>
        public Token(char chToken, CharType chType)
        {
            _token = "";

            _token += chToken;
            _charType = chType;
        }

        /// <summary>
        /// TokenType property: Returns the Character type associate with the token.
        /// </summary>
        public CharType TokenType
        {
            get
            {
                return _charType;
            }
        }


        /// <summary>
        /// ToString() - returns the string name of the token.
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return _token;
        }
    }

}
